package gov.grants.apply.agency.v2;




import gov.grants.apply.services.agencywebservices_v2.AgencyWebServicesPortType;
import gov.grants.apply.services.agencywebservices_v2.AssignAgencyTrackingNumberRequest;
import gov.grants.apply.services.agencywebservices_v2.AssignAgencyTrackingNumberResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;


public class AssignAgencyTrackingNumberClient extends BaseAgencyClient {
	
	private static final String CLASSNAME = AssignAgencyTrackingNumberClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );
		
		try {
			AssignAgencyTrackingNumberClient client = new AssignAgencyTrackingNumberClient();
			
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nTEST FAILED - Exception: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	
	public void makeServiceCall() throws Exception {
		try {
			String ggTrackingNum = getArgMap().get( Globals.GG_TRACKING_NUM_CMD_LINE_KEY );
			log.debug( "ggTrackingNum: " + ggTrackingNum );
			
			String agencyTrackingNum = getArgMap().get( Globals.AGENCY_TRACKING_NUM_CMD_LINE_KEY );
			log.debug( "agencyTrackingNum: " + agencyTrackingNum );
			
			AgencyWebServicesPortType port = getAgencyPort();
			
			if ( StringUtils.isBlank( ggTrackingNum ) ) {
				throw  new Exception( "gg_tracking_num is missing from the command line" );
			}// if
			
			if ( StringUtils.isBlank( agencyTrackingNum ) ) {
				throw  new Exception( "agency_tracking_num is missing from the command line" );
			}// if
			
			AssignAgencyTrackingNumberRequest request = new AssignAgencyTrackingNumberRequest();
			request.setGrantsGovTrackingNumber( ggTrackingNum );
			request.setAgencyTrackingNumber( agencyTrackingNum );
			
			log.debug( "Make service call" );
			AssignAgencyTrackingNumberResponse response = port.assignAgencyTrackingNumber( request );
			
			log.debug( "AssignAgencyTrackingNumber response msg: " + response.getResponseMessage() );
			
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			throw e;
		}// try-catch
		
	}// makeServiceCall
	
	
}
