package gov.grants.apply.agency.v2;


import java.util.ArrayList;
import java.util.List;


import gov.grants.apply.services.agencywebservices_v2.AgencyWebServicesPortType;
import gov.grants.apply.services.agencywebservices_v2.CreateAdobeOpportunityInfo;
import gov.grants.apply.services.agencywebservices_v2.CreateAdobeOpportunityResult;
import gov.grants.apply.services.agencywebservices_v2.CreateAdobeOpportunityRequest;
import gov.grants.apply.services.agencywebservices_v2.CreateAdobeOpportunityResponse;
import gov.grants.apply.system.grantsfundingsynopsis_v2.AdditionalInformation;
import gov.grants.apply.system.grantsfundingsynopsis_v2.AgencyContact;
import gov.grants.apply.system.grantsfundingsynopsis_v2.FundingOppSynopsis20;
import gov.grants.commons.util.FileUtil;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.ximpleware.VTDGen;
import com.ximpleware.VTDNav;

public class CreateAdobeOpportunityClient extends BaseAgencyClient {
	
	private static final String CLASSNAME = CreateAdobeOpportunityClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );

		try {
			CreateAdobeOpportunityClient client = new CreateAdobeOpportunityClient();
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nTEST FAILED - Exception: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	
	public void makeServiceCall() throws Exception {
		String oppXml = null;
		try {
			String oppXmlFilePath = getArgMap().get( Globals.OPP_XML_CMD_LINE_KEY );
			log.debug( "Loading OPP XML File: " + oppXmlFilePath );
			try {
				byte[] fileBytes = FileUtil.readFile( oppXmlFilePath );
				oppXml = new String( fileBytes );
			} catch ( Exception e ) {
				log.error( "Exception caught reading file [" + oppXmlFilePath + "]:" + e.getMessage() );
			}// try-catch
			if ( StringUtils.isBlank( oppXml ) ) {
				String s = "Could not load opportunity XML:" + oppXmlFilePath;
				log.error( s );
				throw new Exception( s );
			}// if
			CreateAdobeOpportunityRequest request = constructCreateAdobeOpportunityRequestFromXml(oppXmlFilePath);
			AgencyWebServicesPortType port = getAgencyPort();
			log.debug( "port: " + port );
			log.debug( "Make service call" );
			CreateAdobeOpportunityResponse response = port.createAdobeOpportunity( request );
			log.debug( "Response object recieved is: " + response.toString());
			log.debug("Response Object contains Error: " + String.valueOf(!response.isSuccess()));
			log.debug("***************************************");
			for( CreateAdobeOpportunityResult result : response.getCreateAdobeOpportunityResult()) {
				String oppId = result.getFundingOpportunityNumber();
				if(result.getErrorDetails() != null) {
					String code = result.getErrorDetails().getCode();
					String msg = result.getErrorDetails().getMessage();
					log.debug("Result for Create Adobe Opportunity with ID: " + oppId + " \nCode: " + code + "\nMessage: " + msg);
				} else {
					log.debug("Result for Create Adobe Opportunity with ID: " + oppId + " - Success!");
				}
			}
			log.debug("***************************************");
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
		
	}// makeServiceCall
	
	
	private CreateAdobeOpportunityRequest constructCreateAdobeOpportunityRequestFromXml( String oppXmlFilePath) 
			throws Exception {
			
			CreateAdobeOpportunityRequest retVal = null;
			try {
	    		retVal = new CreateAdobeOpportunityRequest();
	    		log.debug( "Load XML: " + oppXmlFilePath);
				byte[] xmlBytes = FileUtil.readFile( oppXmlFilePath );
	    		log.debug( "XML bytes loaded: " + xmlBytes.length );
	    		VTDGen vGen = new VTDGen();
	    		vGen.setDoc( xmlBytes );
	    		vGen.parse( false );//set namespace awareness
	    		VTDNav createAdobeOppVN = vGen.getNav();
	    		
	    		if(createAdobeOppVN.toElement( VTDNav.ROOT) && createAdobeOppVN.matchElement(Globals.CREATE_ADOBE_OPP_INFO)) {
	    			log.debug( "adding CreateAdobeOpportunityInfo Object to the request" );
	    			CreateAdobeOpportunityInfo info = new CreateAdobeOpportunityInfo();
		    		List <FundingOppSynopsis20> fundingOppSynopsisList = new ArrayList <FundingOppSynopsis20> (3);

		    		if(createAdobeOppVN.toElement( VTDNav.FIRST_CHILD,Globals.SYNOPSIS_XML)) {
	    				do {
	    					log.debug( "adding FundingOppSynopsis Object to the request" );
		    				FundingOppSynopsis20 fundingOppSynopsis = new FundingOppSynopsis20();

		    				if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_POST_DATE)) {
			    	    		String postingDate = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setPostingDate(postingDate);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    				}
		    	    		
		    				if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_USER_ID)) {
			    	    		String userId = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setUserID(userId);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    				}
		    	    		
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_PASSWORD)) {
			    	    		String password = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setPassword(password);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_INST)) {
			    	    		List <String> fundingInstsList = new ArrayList <String>(3);
		    	    			fundingInstsList.add( createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText()));
		    		    		while ( createAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_FUNDING_INST) ) {
		    		    			fundingInstsList.add( createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText()));
		    		    		}
		    	    			fundingOppSynopsis.getFundingInstrument().addAll(fundingInstsList);
		    	    			createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}

		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_ACT_CAT)) {
			    	    		List <String> fundingActsList = new ArrayList <String>(3);
		    	    			fundingActsList.add( createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText()));
		    		    		while ( createAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_FUNDING_ACT_CAT) ) {
		    		    			fundingActsList.add( createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText()));
		    		    		}
		    	    			fundingOppSynopsis.getFundingActivityCategory().addAll(fundingActsList);
		    	    			createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_OTHER_FUNDING_CAT_EXP)) {
			    	    		String otherFundingCatExp = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setOtherFundingCategoryExplanation( otherFundingCatExp);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_EXPECTED_NUM_OF_AWARDS)) {
			    	    		String expNumOfAwards = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setExpectedNumberOfAwards( expNumOfAwards);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ESTIMATED_FUNDING)) {
			    	    		String estFunding = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setEstimatedFunding( estFunding);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AWARD_CEILING)) {
			    	    		String awardCeiling = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setAwardCeiling(awardCeiling);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AWARD_FLOOR)) {
			    	    		String awardFloor = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setAwardFloor( awardFloor);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}

		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_OPP_NUM)) {
			    	    		String fundingOppNum = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setFundingOpportunityNumber( fundingOppNum);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}

		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_OPP_TITLE)) {
			    	    		String fundingOppTitle = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setFundingOpportunityTitle( fundingOppTitle);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}

		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_OPP_DESC)) {
			    	    		String fundingOppDesc = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setFundingOpportunityDescription( fundingOppDesc);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}

		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ClOSING_DATE)) {
			    	    		String closingDate = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setClosingDate( closingDate);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_CLOSING_DATE_EXP)) {
			    	    		String closingDateExp = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setClosingDateExplanation( closingDateExp);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ARCHIVE_DATE)) {
			    	    		String archiveDate = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setArchiveDate( archiveDate);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_CFDA_NUM)) {
		    		    		List <String> cfdaNumsList = new ArrayList <String>(3);
		    				    cfdaNumsList.add( createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText()));
		    		    		while ( createAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_CFDA_NUM) ) {
		    		    			cfdaNumsList.add( createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText()));
		    		    		}
		    	    			fundingOppSynopsis.getCFDANumber().addAll(cfdaNumsList);
		    	    			createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ELIGIBILE_APP_TYPE)) {
			    	    		List <String> eligibleAppTypesList = new ArrayList <String>(3);
		    	    			eligibleAppTypesList.add( createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText()));
		    		    		while ( createAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_ELIGIBILE_APP_TYPE) ) {
		    		    			eligibleAppTypesList.add( createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText()));
		    		    		}
		    	    			fundingOppSynopsis.getEligibleApplicantTypes().addAll(eligibleAppTypesList);
		    	    			createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_OTHER_ELIGIBILE_APP_EXP)) {
			    	    		String otherEligibileAppExp = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setOtherEligibleApplicantExplanation( otherEligibileAppExp);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_COST_SHARING_OR_MATCHING_REQ)) {
			    	    		String costSharingOrMatchReq = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
			    	    		fundingOppSynopsis.setCostSharingOrMatchingRequirement( costSharingOrMatchReq);
			    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		

		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ADD_INFO)) {
		    	    			AdditionalInformation additionalInfo = new AdditionalInformation();

			    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ADD_INFO_URL)) {
				    	    		String  additionalInfoURL = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
				    	    		additionalInfo.setAdditionalInformationURL(additionalInfoURL);
				    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
			    	    		}

			    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ADD_INFO_URL_DESC)) {
				    	    		String URLDesc = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
				    	    		additionalInfo.setURLDescription(URLDesc);
				    	    		createAdobeOppVN.toElement( VTDNav.PARENT);
			    	    		}
			    	    		fundingOppSynopsis.setAdditionalInformation(additionalInfo);	
		    	    			createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AGENCY_CONTACT)) {
		    		    		List <AgencyContact> agencyContactsList = new ArrayList <AgencyContact>(3);
		    				    do {

		    	    				AgencyContact agencyContact = new AgencyContact();
			        				
			        				if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AGENCY_CONTACT_DESC)) {
				            			String agencyContactDesc = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
				            			agencyContact.setAgencyContactDescription(agencyContactDesc);
				            			createAdobeOppVN.toElement( VTDNav.PARENT);
			        				}
			            			
			            			if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AGENCY_EMAIL_ADDR)) {
				            			String agencyContactEmailAddr = createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
				            			agencyContact.setAgencyEmailAddress( agencyContactEmailAddr);
				            			createAdobeOppVN.toElement( VTDNav.PARENT);
			            			}
			            			
			            			if( createAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AGENCY_EMAIL_DESC)) {
				            			String agencyContactEmailAddrDesc =  createAdobeOppVN.toNormalizedString2( createAdobeOppVN.getText());
				            			agencyContact.setAgencyEmailDescription( agencyContactEmailAddrDesc);
				            			createAdobeOppVN.toElement( VTDNav.PARENT);
			            			}
			    	    			agencyContactsList.add( agencyContact);
		    	    			}while(createAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_AGENCY_CONTACT));
		    	    			fundingOppSynopsis.getAgencyContact().addAll(agencyContactsList);
		    	    			createAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		fundingOppSynopsisList.add(fundingOppSynopsis);
		    			} while (createAdobeOppVN.toElement( VTDNav.NEXT_SIBLING,Globals.SYNOPSIS_XML));
		    		
	    				info.getFundingOppSynopsis20().addAll(fundingOppSynopsisList);
		    			retVal.setCreateAdobeOpportunityInfo(info);
	    			
		    		} else {
		    			String s = "Cannot find any FundingOppSynopsis_2_0 Element in the input XML [" + oppXmlFilePath + "]";
						throw new Exception( s );
		    		}
	    		} else {
					String s = "Cannot find CreateAdobeOpportunityInfo Root Element in the input XML [" + oppXmlFilePath + "]";
					throw new Exception( s );
	    		}
	    			
	    	} catch ( Exception e ) {
	    		log.error( "Client Exception occurred while parsing input file xml [" + oppXmlFilePath + "]: " + e.getMessage() );
	    		log.error( "Check your Input file and make sure it is in compliance with the published schema of the current version.");
	    		throw e;
	    	}// try-catch
			return retVal;
		}


}
