package gov.grants.apply.agency.v2;


import java.io.File;
import java.util.List;
import gov.grants.apply.services.agencywebservices_v2.AgencyWebServicesPortType;
import gov.grants.apply.services.agencywebservices_v2.GetApplicationRequest;
import gov.grants.apply.services.agencywebservices_v2.GetApplicationResponse;
import gov.grants.apply.system.grantscommonelements_v1.Attachment;
import gov.grants.commons.s2s.util.SoapUtil;
import gov.grants.commons.util.FileUtil;
import org.apache.log4j.Logger;
import com.sun.xml.ws.developer.StreamingDataHandler;


/* 
 * GetApplicationClient 
 * url=https://devweb.grants.gov:8002/grantsws-applicant/services/ApplicantIntegrationSoapPort 
 * processid=GRANTxxxxxxxx
 */

public class GetApplicationClient extends BaseAgencyClient {
	
	private static final String CLASSNAME = GetApplicationClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );
		
		try {
			GetApplicationClient client = new GetApplicationClient();
			
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nTEST FAILED - Exception: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	
	public void makeServiceCall() throws Exception {
		try {
			String trackingNum = getArgMap().get( Globals.GG_TRACKING_NUM_CMD_LINE_KEY );
			log.debug( "trackingNum: " + trackingNum );
			
			String downloadDir = SoapUtil.getProperty( Globals.DOWNLOAD_DIR__KEY );
			log.debug( "downloadDir: " + downloadDir );
			
			String appDownloadDir = downloadDir + "/" + trackingNum;
			File dir = new File( appDownloadDir );
			if ( !dir.exists() && !dir.mkdirs() ) {
				log.error( "Could not create dir: " + appDownloadDir );
				throw new Exception( "Could not create dir: " + appDownloadDir );
			}// if
			
			log.debug( "created dir: " + appDownloadDir );
			AgencyWebServicesPortType port = getAgencyPort();
			GetApplicationRequest request = new GetApplicationRequest();
			request.setGrantsGovTrackingNumber( trackingNum );
			GetApplicationResponse response = port.getApplication( request );
			String xmlStr = response.getSubmittedApplication();
			log.debug( "application xml: " + xmlStr );
			
			String filename = appDownloadDir + "/" + trackingNum + ".xml";
			FileUtil.bytesToDisk( xmlStr.getBytes(), filename );
			// MAMER: We no longer use SOAP Attachment but rather data handler so we iterate over the Data handler 
			// coming from the Response object to save them to disk.
			int count = writeAttachmentsToDisk( appDownloadDir, response.getAttachment());
			log.debug("Total number of Attachments saved is: " + count);
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
	}// makeServiceCall
	
	
	private int writeAttachmentsToDisk( String appDownloadDir, List <Attachment> attachments ) 
			throws Exception {
				try {
					int count = 0;				
					if ( attachments != null && attachments.size() > 0 ) {
						count = attachments.size();
						StreamingDataHandler dh = null;
						String contentId = null;
						StringBuilder fileName = new StringBuilder();
						// Note for developer: Make sure that you call the moveTo() method on the Data handler of each attachment
						// in the same exact order failure to do so may cause the retrieval of the attachment to fail!
						for ( int i= 0; i < attachments.size(); i++) {
							dh = (StreamingDataHandler) attachments.get(i).getFileDataHandler();
							contentId = attachments.get(i).getFileContentId();
							int index = contentId.indexOf( Globals.CID_PREFIX );
							if ( index > -1 ) {
								log.debug( "stripping CID prefix from contentId: " + contentId );
								contentId = contentId.substring( index + Globals.CID_PREFIX_LENGTH );
							}// if
							log.debug( "final contentId: " + contentId );
							log.debug( "content type: " + dh.getContentType() );
							fileName.append( appDownloadDir ).append( "/" ).append( contentId );
							// test.txt
//							log.debug( "last index of: " + contentId.lastIndexOf( "." ) );
//							log.debug( "required file ext beginning: " + ( contentId.length() - 4 ) );
//							if ( contentId.lastIndexOf( "." ) != ( contentId.length() - 4 ) ) {
//								log.debug( "appending '.pdf' to FIF file name" );
//								fileName.append( ".pdf" );
//							}// if
							log.debug( "Start streaming Binary Data of attachment to the Client from the Server with cid: " + contentId);
							File f = new File(fileName.toString());
						    log.debug( "dh.moveTo()" );
						    dh.moveTo( f );
						    log.debug( "dh.close()" );
						    dh.close();
						    log.debug( "Finished Streaming Binary Data of attachment to the Client from the Server with cid: " + contentId);
							fileName.setLength( 0 );
						}// for
					}// if
					return count;
				} catch ( Exception e ) {
					log.error( "Exception caught retrieving attachments: " + e.getMessage() );
					e.printStackTrace();
					throw e;
				}// try-catch
			}// writeAttachmentsToDisk
			
	
	
}
