package gov.grants.apply.agency.v2;



import gov.grants.apply.services.agencywebservices_v2.AgencyWebServicesPortType;
import gov.grants.apply.services.agencywebservices_v2.GetApplicationListRequest;
import gov.grants.apply.services.agencywebservices_v2.GetApplicationListResponse;
import gov.grants.apply.services.agencywebservices_v2.GetApplicationListResponse.ApplicationInfo;
import gov.grants.apply.system.grantscommonelements_v1.ApplicationFilter;
import gov.grants.business.util.CommonConstants;
import org.apache.commons.lang.builder.ToStringBuilder;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;



public class GetApplicationListClient extends BaseAgencyClient {
	
	private static final String CLASSNAME = GetApplicationListClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	private static final String EMPTY_FILTER_VALUE = "EMPTY";
	private static final String BLANK_FILTER_VALUE = "BLANK";
	
//	private Map<String, String> statusMap = new HashMap<String, String>();
	
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );
		
		try {
			GetApplicationListClient client = new GetApplicationListClient();
//			client.init();// init status map
			
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nTEST FAILED - Exception: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	
	public void makeServiceCall() throws Exception {
		try {
			String status = getArgMap().get( Globals.STATUS_CMD_LINE_KEY );
			log.debug( "status: " + status );
			
			AgencyWebServicesPortType port = getAgencyPort();
			
			if ( StringUtils.isBlank( status ) ) {
				log.debug( "check for following status: empty, " + CommonConstants.READY_FOR_DOWNLOAD + ", " + CommonConstants.RECEIVED_BY_AGENCY + ", and blank" );
				getApplicationListWithEmptyFilter( port );
				getApplicationListWithStatus( port, CommonConstants.READY_FOR_DOWNLOAD );
				getApplicationListWithStatus( port, CommonConstants.RECEIVED_BY_AGENCY );
				getApplicationListWithStatus( port, "" );
			} else {
				if ( EMPTY_FILTER_VALUE.equals( status.toUpperCase() ) ) {
					getApplicationListWithEmptyFilter( port );
					
				} else if ( BLANK_FILTER_VALUE.equals( status.toUpperCase() ) ) {
					getApplicationListWithStatus( port, "" );
					
				} else {
					getApplicationListWithStatus( port, status );
				}// if-else
			}// if-else
			
			
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
		
	}// makeServiceCall

	
	
	public void getApplicationListWithEmptyFilter( AgencyWebServicesPortType port ) 
	throws Exception {
		try {
			log.debug( "\n\n======= Start Get Application List With Empty Filter =========" );
			GetApplicationListRequest request = new GetApplicationListRequest();
			List<ApplicationFilter> filters = request.getApplicationFilter();
			ApplicationFilter filter = new ApplicationFilter();
			filters.add( filter );
			
			GetApplicationListResponse response = port.getApplicationList(request);
			List <ApplicationInfo> list = response.getApplicationInfo();
			printList( list );
			log.debug( "Total Available Application Count: " + response.getAvailableApplicationNumber());
			
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
	}// getApplicationListWithEmptyFilter
	
	
	public void getApplicationListWithStatus( AgencyWebServicesPortType port, String status ) 
	throws Exception {
		try {
			log.debug( "\n\n======= Start Get Application List With [" + status + "] Status Filter ========" );
			
			GetApplicationListRequest request = new GetApplicationListRequest();
			List<ApplicationFilter> filters = request.getApplicationFilter();
			ApplicationFilter filter = new ApplicationFilter();
			filter.setFilter( "Status");
			filter.setFilterValue( status );
			filters.add( filter );
			
			GetApplicationListResponse response = port.getApplicationList(request );
			List <ApplicationInfo> list = response.getApplicationInfo();
			printList( list );
			log.debug( "Total Available Application Count: " + response.getAvailableApplicationNumber());
			
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
	}// getApplicationListWithStatus
	
	
	
	   
	public void printList( List<ApplicationInfo> list ) {
		if ( list == null || list.size() == 0 ) {
			log.debug( "Application list is empty." );
				return;
		}// if
	         
		log.debug( "Received " + list.size() + " applications in list" );
		StringBuilder sb = new StringBuilder().append( "\n" );
		int count = 0;
		for ( ApplicationInfo info : list ) {
			log.debug( "info: " + ToStringBuilder.reflectionToString( info ) );
			count++;
			sb.append( "Application list element " )
				.append( count )
				.append( ": " )
				.append( info.getGrantsGovTrackingNumber() )
				.append( "--- Status: " )
				.append( info.getGrantsGovApplicationStatus() )
				.append( "\n" );
		}// for-each
			
		log.debug( sb.toString() );
	}// printList
	
	

	
}
