package gov.grants.apply.agency.v2;

import java.util.ArrayList;
import java.util.List;
import javax.activation.DataHandler;
import javax.activation.FileDataSource;

import gov.grants.apply.agency.v2.Globals;
import gov.grants.apply.services.agencywebservices_v2.AgencyWebServicesPortType;
import gov.grants.apply.services.agencywebservices_v2.ManagePackageRequest;
import gov.grants.apply.services.agencywebservices_v2.ManagePackageResponse;
import gov.grants.apply.system.agencymanagepackage_v1.CreateOpportunityInfo;
import gov.grants.apply.system.agencymanagepackage_v1.CreatePackageInfo;
import gov.grants.apply.system.agencymanagepackage_v1.CreatePackageResult;
import gov.grants.apply.system.agencymanagepackage_v1.DeletePackageInfo;
import gov.grants.apply.system.agencymanagepackage_v1.DeletePackageResult;
import gov.grants.apply.system.agencymanagepackage_v1.EmailInfo;
import gov.grants.apply.system.agencymanagepackage_v1.UpdatePackageInfo;
import gov.grants.apply.system.agencymanagepackage_v1.UpdatePackageResult;
import gov.grants.apply.system.grantscommonelements_v1.CompetitionInfo;
import gov.grants.apply.system.grantscommonelements_v1.InstructionFileInfo;
import gov.grants.apply.system.grantscommontypes_v1.OpportunityCategoryType;
import gov.grants.commons.util.FileUtil;
import org.apache.log4j.Logger;
import com.ximpleware.VTDGen;
import com.ximpleware.VTDNav;

public class ManagePackageClient extends BaseAgencyClient {
	
	private static final String CLASSNAME = ManagePackageClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );

		try {
			ManagePackageClient client = new ManagePackageClient();
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nTEST FAILED - Exception: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	
	public void makeServiceCall() throws Exception {
		
		try {
			String configXml = getArgMap().get( Globals.CONFIG_XML_CMD_LINE_KEY );
			loadConfigXmlDoc( configXml );
			configVN.toElement( VTDNav.FIRST_CHILD, CLASSNAME );// navigate to client element
			
			configVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_DIR_ELEMENT_NAME );
			String managePkgDir = configVN.toNormalizedString2( configVN.getText() );
			log.debug( "managePkgDir: " + managePkgDir );
			configVN.toElement( VTDNav.PARENT );
			configVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_XML_ELEMENT_NAME );
			String managePkgXmlFileName = configVN.toNormalizedString2( configVN.getText() );
			log.debug( "xmlFileName: " + managePkgXmlFileName );
			log.debug( "Constructing ManagePackageRequest from xml file: " + managePkgDir + managePkgXmlFileName );
			ManagePackageRequest request = constructManagePackageRequestFromXml(managePkgDir, managePkgXmlFileName);
	
			
			
			// create service
			AgencyWebServicesPortType port = getAgencyPort();
			log.debug( "port: " + port );
			log.debug( "Make service call" );
			ManagePackageResponse response = port.managePackage(request);
			log.debug( "Response object recieved is: " + response.toString());
			log.debug("Response Object contains Error: " + String.valueOf(!response.isSuccess()));
				log.debug("***************************************");
				for( CreatePackageResult result : response.getCreatePackageResult()) {
					String competitionId = result.getCompetitionID();
					String cfda = result.getCFDANumber();
					String logMsg = "Result for Create Package with (Competition ID: " + competitionId +" /CFDA Number: " + cfda + ") ";
					if(result.getErrorDetails() != null) {
						String code = result.getErrorDetails().getCode();
						String msg = result.getErrorDetails().getMessage();
						log.debug(logMsg + " \nCode: " + code + "\nMessage: " + msg);
					} else {
						log.debug( logMsg + " - Success!");

					}
				}
				for( UpdatePackageResult result : response.getUpdatePackageResult()) {
					String competitionId = result.getCompetitionID();
					String cfda = result.getCFDANumber();
					String logMsg = "Result for Update Package with (Competition ID: " + competitionId +" /CFDA Number: " + cfda + ") ";
					if(result.getErrorDetails() != null) {
						String code = result.getErrorDetails().getCode();
						String msg = result.getErrorDetails().getMessage();
						log.debug(logMsg + " \nCode: " + code + "\nMessage: " + msg);
					} else {
						log.debug( logMsg + " - Success!");
					}
				}
				for( DeletePackageResult result : response.getDeletePackageResult()) {
					String competitionId = result.getCompetitionID();
					String cfda = result.getCFDANumber();
					String logMsg = "Result for Delete Package with (Competition ID: " + competitionId +" /CFDA Number: " + cfda + ") ";
					if(result.getErrorDetails() != null) {
						String code = result.getErrorDetails().getCode();
						String msg = result.getErrorDetails().getMessage();
						log.debug(logMsg + " \nCode: " + code + "\nMessage: " + msg);
					} else {
						log.debug( logMsg + " - Success!");
					}
				}
				log.debug("***************************************");
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
		
	}// makeServiceCall
	
	private ManagePackageRequest constructManagePackageRequestFromXml( String managePkgDir, String managePkgXmlFileName) 
		throws Exception {
		
		ManagePackageRequest retVal = null;
		String managePkgXmlFilePath = managePkgDir + managePkgXmlFileName;
		try {
    		retVal = new ManagePackageRequest();
    		log.debug( "Load XML: " + managePkgDir + managePkgXmlFileName );
			byte[] xmlBytes = FileUtil.readFile( managePkgXmlFilePath );
    		log.debug( "XML bytes loaded: " + xmlBytes.length );
    		VTDGen vGen = new VTDGen();
    		vGen.setDoc( xmlBytes );
    		vGen.parse( false );//set namespace awareness
    		VTDNav managePackageVN = vGen.getNav();
    		// MAMER: We need to parse the Manage Package XML to create the necessary ManagePackageRequest.
    		// navigate to ManagePackage element
    		if( managePackageVN.toElement( VTDNav.ROOT) && managePackageVN.matchElement(Globals.MANAGE_PKG_ROOT)) {
    		
	    		if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_USER_ID)) {
		    		String userId = managePackageVN.toNormalizedString2( managePackageVN.getText());
		    		retVal.setUserID(userId);
		    		managePackageVN.toElement( VTDNav.PARENT);
		    	}
	    		
	    		if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_OPP_NUM)) {
		    		String oppNum = managePackageVN.toNormalizedString2( managePackageVN.getText());
		    		retVal.setFundingOpportunityNumber(oppNum);
		       		managePackageVN.toElement( VTDNav.PARENT);
	    		}
	    
	       		if (managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CREATE_OPP_INFO)) {
	    			CreateOpportunityInfo createOppInfo = new CreateOpportunityInfo();
	
	    			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_OPP_TITLE)) {
			    		String oppTitle = managePackageVN.toNormalizedString2( managePackageVN.getText());
			    		createOppInfo.setFundingOpportunityTitle(oppTitle);
			    		managePackageVN.toElement( VTDNav.PARENT);
	    			}
		    		
		    		if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_OPP_CAT)) {
			    		String oppCat = managePackageVN.toNormalizedString2( managePackageVN.getText());
			    		createOppInfo.setOpportunityCategory( OpportunityCategoryType.valueOf(oppCat));
			    		managePackageVN.toElement( VTDNav.PARENT);
		    		}
		    		
		    		if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_OPP_CAT_EXP)) {
			    		String oppCatExp = managePackageVN.toNormalizedString2( managePackageVN.getText());
			    		createOppInfo.setOpportunityCategoryExplanation(oppCatExp);
			    		managePackageVN.toElement( VTDNav.PARENT);
		    		}
		    		
		    		if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CFDA_NUM)) {
			    		List <String> cdfaNumsList = new ArrayList <String>(3);
		    			cdfaNumsList.add( managePackageVN.toNormalizedString2( managePackageVN.getText()));
			    		while ( managePackageVN.toElement( VTDNav.NEXT_SIBLING, Globals.MANAGE_PKG_CFDA_NUM) ) {
			    			cdfaNumsList.add( managePackageVN.toNormalizedString2( managePackageVN.getText()));
			    		}
		    			createOppInfo.getCFDANumber().addAll(cdfaNumsList);
			    		managePackageVN.toElement( VTDNav.PARENT);
		    		}
		    		// Finally we set the CreateOpportunityInfo object in the request.
		    		retVal.setCreateOpportunityInfo(createOppInfo);
		    		// Move back to the Parent Level.
		    		managePackageVN.toElement( VTDNav.PARENT);
	   			}
	    			// Parse the Create Package Info Elements.
	    			List <CreatePackageInfo> createPackageInfoList = new ArrayList <CreatePackageInfo> (3);
	    			if(managePackageVN.toElement( VTDNav.FIRST_CHILD,Globals.MANAGE_PKG_CREATE_PKG_INFO)) {
		    			do {
		    				log.debug( "adding CreatePackageInfo Object to the request" );
		    				CreatePackageInfo createPackageInfo = new CreatePackageInfo();
		
		    				if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_ELECT_REQ)) {
			        			String electronicSignatureRequired = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setElectronicSignatureRequired( electronicSignatureRequired);
			        			managePackageVN.toElement( VTDNav.PARENT);
		    				}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_EXP_APP_COUNT)) {
			        			String expAppCount = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setExpectedApplicationCount(expAppCount);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CLOSING_DATE)) {
			        			String closingDate = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setClosingDate(closingDate);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_TMPL_NAME)) {
			        			String templateName = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setTemplateName(templateName);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if(managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_INST_FILE_INFO)) {
		        				InstructionFileInfo instFileInfo = new InstructionFileInfo();
		        				String fullInstFilePath = managePkgDir; 	
		        				if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_FILE_NAME)) {
			            			String instFileName = managePackageVN.toNormalizedString2( managePackageVN.getText());
			            			instFileInfo.setFileName( instFileName);
			            			fullInstFilePath += instFileName;
			            			managePackageVN.toElement( VTDNav.PARENT);
		        				}
		        				
		            			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_FILE_EXT)) {
			            			String instFileExt = managePackageVN.toNormalizedString2( managePackageVN.getText());
			            			instFileInfo.setFileExtension(instFileExt);
			            			managePackageVN.toElement( VTDNav.PARENT);
		            			}
		            			
		            			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CID)) {
			            			String cid =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			            			instFileInfo.setFileContentId(cid);
			        				managePackageVN.toElement( VTDNav.PARENT);	
		            			}
	
		            			// Construct the Data Handler for the instruction file.
		            			instFileInfo.setFileDataHandler( new DataHandler(new FileDataSource(fullInstFilePath)));            			
		            			createPackageInfo.setInstructionFileInfo( instFileInfo);
		        				managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_OPEN_TO_APP_TYPE)) {
			        			String openToApplicantType =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setOpenToApplicant( openToApplicantType);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
	
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CFDA_NUM)) {
			        			String cfdaNum =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setCFDANumber(cfdaNum);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if(managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_COMP_INFO)) {
		        				CompetitionInfo compInfo = new CompetitionInfo();
		        				
		            			if( managePackageVN.toElement(VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_COMP_ID)) {
			        				String compId = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        				compInfo.setCompetitionID(compId);
			        				managePackageVN.toElement( VTDNav.PARENT);
		            			}
		            			
		        				if( managePackageVN.toElement(VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_COMP_TITLE)) {
			        				String compTitle = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        				compInfo.setCompetitionTitle(compTitle);
			        				managePackageVN.toElement( VTDNav.PARENT);	
		        				}
		        				
		        				createPackageInfo.setCompetitionInfo(compInfo);
		        				managePackageVN.toElement( VTDNav.PARENT);
		        			}
		
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_EXP_APP_SIZE)) {
			        			String expectedAppSize =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setExpectedApplicationSizeMB(expectedAppSize);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
	
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_OPENING_DATE)) {
			        			String openingDate =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setOpeningDate(openingDate);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_GRACE_PERIOD)) {
			        			String gracePeriod =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setGracePeriodDays(gracePeriod);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CONTACT_INFO)) {
			        			String agencyContactInfo =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			createPackageInfo.setAgencyContactInfo(agencyContactInfo);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			createPackageInfoList.add(createPackageInfo);
	
		    			} while ( managePackageVN.toElement( VTDNav.NEXT_SIBLING,Globals.MANAGE_PKG_CREATE_PKG_INFO));   // End do-While
		    			retVal.getCreatePackageInfo().addAll(createPackageInfoList);
		    			managePackageVN.toElement( VTDNav.PARENT);
	    			}
	    			
	    			// Parse the Update Package Info Elements.
	    			List <UpdatePackageInfo> updatePackageInfoList = new ArrayList <UpdatePackageInfo> (3);
	    			if(managePackageVN.toElement( VTDNav.FIRST_CHILD,Globals.MANAGE_PKG_UPDATE_PKG_INFO)) {
		    			do {
		    				log.debug( "adding UpdatePackageInfo Object to the request" );
		    				UpdatePackageInfo updatePackageInfo = new UpdatePackageInfo();
		
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CFDA_NUM)) {
			        			String cfdaNum =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setCFDANumber(cfdaNum);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		       				
		        			if( managePackageVN.toElement(VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_COMP_ID)) {
			       				String compId = managePackageVN.toNormalizedString2( managePackageVN.getText());
			       				updatePackageInfo.setCompetitionID(compId);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_COMP_TITLE)) {
			        			String compTitle = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setCompetitionTitle(compTitle);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_ELECT_REQ)) {
			        			String electronicSignatureRequired = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setElectronicSignatureRequired( electronicSignatureRequired);
			        			managePackageVN.toElement( VTDNav.PARENT);
			        		}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_EXP_APP_COUNT)) {
			        			String expAppCount = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setExpectedApplicationCount(expAppCount);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CLOSING_DATE)) {
			        			String closingDate = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setClosingDate(closingDate);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_TMPL_NAME)) {
			        			String templateName = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setTemplateName(templateName);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if(managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_INST_FILE_INFO)) {
		        				InstructionFileInfo instFileInfo = new InstructionFileInfo();
		        				String fullInstFilePath = managePkgDir; 	
		        				if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_FILE_NAME)) {
			            			String instFileName = managePackageVN.toNormalizedString2( managePackageVN.getText());
			            			instFileInfo.setFileName( instFileName);
			            			fullInstFilePath += instFileName;
			            			managePackageVN.toElement( VTDNav.PARENT);
		        				}
		        				
		            			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_FILE_EXT)) {
			            			String instFileExt = managePackageVN.toNormalizedString2( managePackageVN.getText());
			            			instFileInfo.setFileExtension(instFileExt);
			            			managePackageVN.toElement( VTDNav.PARENT);
		            			}
		            			
		            			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CID)) {
			            			String cid =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			            			instFileInfo.setFileContentId(cid);
			        				managePackageVN.toElement( VTDNav.PARENT);	
		            			}
	
		            			// Construct the Data Handler for the instruction file.
		            			instFileInfo.setFileDataHandler( new DataHandler(new FileDataSource(fullInstFilePath)));            			
		            			updatePackageInfo.setInstructionFileInfo( instFileInfo);
		        				managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_OPEN_TO_APP_TYPE)) {
			        			String openToApplicantType =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setOpenToApplicant( openToApplicantType);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_EXP_APP_SIZE)) {
			        			String expectedAppSize =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setExpectedApplicationSizeMB(expectedAppSize);
			        			managePackageVN.toElement( VTDNav.PARENT);
		    				}
		    			
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_OPENING_DATE)) {
			        			String openingDate =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setOpeningDate(openingDate);
			        			managePackageVN.toElement( VTDNav.PARENT);
	    					}	
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_GRACE_PERIOD)) {
			        			String gracePeriod =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setGracePeriodDays(gracePeriod);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		        			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CONTACT_INFO)) {
			        			String agencyContactInfo =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			updatePackageInfo.setAgencyContactInfo(agencyContactInfo);
			        			managePackageVN.toElement( VTDNav.PARENT);
		        			}
		
		        			if(managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_EMAIL_INFO)) {
		        				EmailInfo emailInfo = new EmailInfo();
		        			
		        				if(managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_SEND_EMAIL)) {
			        				String sendEmail = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        				emailInfo.setSendChangeNotificationEmail(sendEmail);
			        				managePackageVN.toElement( VTDNav.PARENT);
			        			}
	
			        			if( managePackageVN.toElement(VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_EMAIL_COMMENTS)) {
			        				String emailComments = managePackageVN.toNormalizedString2( managePackageVN.getText());
			        				emailInfo.setModificationComments(emailComments);
			        				managePackageVN.toElement( VTDNav.PARENT);	
			        			}
	
		        				updatePackageInfo.setEmailInfo(emailInfo);
		        			}
		        			updatePackageInfoList.add(updatePackageInfo);
		        			managePackageVN.toElement( VTDNav.PARENT);
		    			} while ( managePackageVN.toElement( VTDNav.NEXT_SIBLING,Globals.MANAGE_PKG_UPDATE_PKG_INFO));
		    			retVal.getUpdatePackageInfo().addAll(updatePackageInfoList);
		    			managePackageVN.toElement( VTDNav.PARENT);
	    			}
	    			
	    			// Parse the Delete Package Info Elements.
	    			List <DeletePackageInfo> deletePackageInfoList = new ArrayList <DeletePackageInfo> (3);
	    			if(managePackageVN.toElement( VTDNav.FIRST_CHILD,Globals.MANAGE_PKG_DELETE_PKG_INFO)) {
		    			do {
		    				log.debug( "adding DeletePackageInfo Object to the request" );
		    				DeletePackageInfo deletePackageInfo = new DeletePackageInfo();
	
		    				if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_CFDA_NUM)) {
			        			String cfdaNum =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			deletePackageInfo.setCFDANumber(cfdaNum);
			          			managePackageVN.toElement( VTDNav.PARENT);
		    				}
		        			
		          			if( managePackageVN.toElement( VTDNav.FIRST_CHILD, Globals.MANAGE_PKG_COMP_ID)) {
			        			String compId =  managePackageVN.toNormalizedString2( managePackageVN.getText());
			        			deletePackageInfo.setCompetitionID(compId);
			        			managePackageVN.toElement( VTDNav.PARENT);
		          			}	        			
		        			deletePackageInfoList.add(deletePackageInfo);
		    			
		    			} while ( managePackageVN.toElement( VTDNav.NEXT_SIBLING,Globals.MANAGE_PKG_DELETE_PKG_INFO));
		    			retVal.getDeletePackageInfo().addAll(deletePackageInfoList);
		    			log.debug( "Successfully added all XML Objects to the request" );
	    			}
    			} else {
					String s = "Cannot find ManagePackage Root Element in the input XML [" + managePkgXmlFilePath + "]";
					throw new Exception( s );
    			}
    		} catch ( Exception e ) {
	    		log.error( "Client Exception occurred while parsing input xml [" + managePkgXmlFilePath + "]: " + e.getMessage() );
	    		log.error( "Check your Input file and make sure it is in compliance with the published schema of the current version.");
	    		throw e;
	    	}// try-catch
			return retVal;
	}
}

