package gov.grants.apply.agency.v2;

import java.util.ArrayList;
import java.util.List;


import gov.grants.apply.services.agencywebservices_v2.AgencyWebServicesPortType;
import gov.grants.apply.services.agencywebservices_v2.UpdateAdobeOpportunityInfo;
import gov.grants.apply.services.agencywebservices_v2.UpdateAdobeOpportunityRequest;
import gov.grants.apply.services.agencywebservices_v2.UpdateAdobeOpportunityResponse;
import gov.grants.apply.services.agencywebservices_v2.UpdateAdobeOpportunityResult;
import gov.grants.apply.system.grantsfundingsynopsis_v2.AdditionalInformation;
import gov.grants.apply.system.grantsfundingsynopsis_v2.AgencyContact;
import gov.grants.apply.system.grantsfundingsynopsis_v2.FundingOppModSynopsis20;
import gov.grants.commons.util.FileUtil;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.ximpleware.VTDGen;
import com.ximpleware.VTDNav;

public class UpdateAdobeOpportunityClient extends BaseAgencyClient {
	
	private static final String CLASSNAME = UpdateAdobeOpportunityClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );

		try {
			UpdateAdobeOpportunityClient client = new UpdateAdobeOpportunityClient();
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nTEST FAILED - Exception: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	
	public void makeServiceCall() throws Exception {
		String oppXml = null;
		try {
			String oppXmlFilePath = getArgMap().get( Globals.OPP_XML_CMD_LINE_KEY );
			log.debug( "Loading OPP XML File: " + oppXmlFilePath );
			try {
				byte[] fileBytes = FileUtil.readFile( oppXmlFilePath );
				oppXml = new String( fileBytes );
			} catch ( Exception e ) {
				log.error( "Exception caught reading file [" + oppXmlFilePath + "]:" + e.getMessage() );
			}// try-catch
			if ( StringUtils.isBlank( oppXml ) ) {
				String s = "Could not load opportunity XML:" + oppXmlFilePath;
				log.error( s );
				throw new Exception( s );
			}// if
			UpdateAdobeOpportunityRequest request = constructUpdateAdobeOpportunityRequestFromXml(oppXmlFilePath);
			AgencyWebServicesPortType port = getAgencyPort();
			log.debug( "port: " + port );
			log.debug( "Make service call" );
			UpdateAdobeOpportunityResponse response = port.updateAdobeOpportunity( request );
			log.debug( "Response object recieved is: " + response.toString());
			log.debug("Response Object contains Error: " + String.valueOf(!response.isSuccess()));
			log.debug("***************************************");
			for( UpdateAdobeOpportunityResult result : response.getUpdateAdobeOpportunityResult()) {
				String oppId = result.getFundingOpportunityNumber();
				if(result.getErrorDetails() != null) {
					String code = result.getErrorDetails().getCode();
					String msg = result.getErrorDetails().getMessage();
					log.debug("Result for Update Adobe Opportunity with ID: " + oppId + " \nCode: " + code + "\nMessage: " + msg);
				} else {
					log.debug("Result for Update Adobe Opportunity with ID: " + oppId + " - Success!");
				}
			}
			log.debug("***************************************");
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
		
	}// makeServiceCall
	
	
	private UpdateAdobeOpportunityRequest constructUpdateAdobeOpportunityRequestFromXml( String oppXmlFilePath) 
			throws Exception {
			
			UpdateAdobeOpportunityRequest retVal = null;
			try {
	    		retVal = new UpdateAdobeOpportunityRequest();
	    		log.debug( "Load XML: " + oppXmlFilePath);
				byte[] xmlBytes = FileUtil.readFile( oppXmlFilePath );
	    		log.debug( "XML bytes loaded: " + xmlBytes.length );
	    		VTDGen vGen = new VTDGen();
	    		vGen.setDoc( xmlBytes );
	    		vGen.parse( false );//set namespace awareness
	    		VTDNav updateAdobeOppVN = vGen.getNav();

	    		if(updateAdobeOppVN.toElement( VTDNav.ROOT) && updateAdobeOppVN.matchElement(Globals.UPDATE_ADOBE_OPP_INFO)) {
	    			log.debug( "adding UpdateAdobeOpportunityInfo Object to the request" );
	    			UpdateAdobeOpportunityInfo info = new UpdateAdobeOpportunityInfo();
		    		List <FundingOppModSynopsis20> fundingOppModSynopsisList = new ArrayList <FundingOppModSynopsis20> (3);

	    			if(updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD,Globals.SYNOPSIS_MOD_XML)) {
	    				do {
	    					log.debug( "adding FundingOppModSynopsis Object to the request" );
	    					FundingOppModSynopsis20 fundingOppModSynopsis = new FundingOppModSynopsis20();
		    				
		    				if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_POST_DATE)) {
			    	    		String postingDate = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setPostingDate(postingDate);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    				}
		    	    		
		    				if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_USER_ID)) {
			    	    		String userId = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setUserID(userId);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    				}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_PASSWORD)) {
			    	    		String password = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setPassword(password);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_INST)) {
			    	    		List <String> fundingInstsList = new ArrayList <String>(3);
		    	    			fundingInstsList.add( updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText()));
		    		    		while ( updateAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_FUNDING_INST) ) {
		    		    			fundingInstsList.add( updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText()));
		    		    		}
		    	    			fundingOppModSynopsis.getFundingInstrument().addAll(fundingInstsList);
		    	    			updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}

		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_ACT_CAT)) {
			    	    		List <String> fundingActsList = new ArrayList <String>(3);
		    	    			fundingActsList.add( updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText()));
		    		    		while ( updateAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_FUNDING_ACT_CAT) ) {
		    		    			fundingActsList.add( updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText()));
		    		    		}
		    	    			fundingOppModSynopsis.getFundingActivityCategory().addAll(fundingActsList);
		    	    			updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_OTHER_FUNDING_CAT_EXP)) {
			    	    		String otherFundingCatExp = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setOtherFundingCategoryExplanation( otherFundingCatExp);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_EXPECTED_NUM_OF_AWARDS)) {
			    	    		String expNumOfAwards = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setExpectedNumberOfAwards( expNumOfAwards);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ESTIMATED_FUNDING)) {
			    	    		String estFunding = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setEstimatedFunding( estFunding);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AWARD_CEILING)) {
			    	    		String awardCeiling = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setAwardCeiling(awardCeiling);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AWARD_FLOOR)) {
			    	    		String awardFloor = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setAwardFloor( awardFloor);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_OPP_NUM)) {
			    	    		String fundingOppNum = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setFundingOpportunityNumber( fundingOppNum);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_OPP_TITLE)) {
			    	    		String fundingOppTitle = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setFundingOpportunityTitle( fundingOppTitle);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_FUNDING_OPP_DESC)) {
			    	    		String fundingOppDesc = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setFundingOpportunityDescription( fundingOppDesc);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ClOSING_DATE)) {
			    	    		String closingDate = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setClosingDate( closingDate);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_CLOSING_DATE_EXP)) {
			    	    		String closingDateExp = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setClosingDateExplanation( closingDateExp);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ARCHIVE_DATE)) {
			    	    		String archiveDate = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setArchiveDate( archiveDate);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_CFDA_NUM)) {
			    	    		List <String> cfdaNumsList = new ArrayList <String>(3);
		    	    			cfdaNumsList.add( updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText()));
		    		    		while ( updateAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_CFDA_NUM) ) {
		    		    			cfdaNumsList.add( updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText()));
		    		    		}
		    	    			fundingOppModSynopsis.getCFDANumber().addAll(cfdaNumsList);
		    	    			updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ELIGIBILE_APP_TYPE)) {
			    	    		List <String> eligibleAppTypesList = new ArrayList <String>(3);
		    	    			eligibleAppTypesList.add( updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText()));
		    		    		while ( updateAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_ELIGIBILE_APP_TYPE) ) {
		    		    			eligibleAppTypesList.add( updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText()));
		    		    		}
		    	    			fundingOppModSynopsis.getEligibleApplicantTypes().addAll(eligibleAppTypesList);
		    	    			updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_OTHER_ELIGIBILE_APP_EXP)) {
			    	    		String otherEligibileAppExp = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setOtherEligibleApplicantExplanation( otherEligibileAppExp);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    				
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_COST_SHARING_OR_MATCHING_REQ)) {
			    	    		String costSharingOrMatchReq = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setCostSharingOrMatchingRequirement( costSharingOrMatchReq);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}

		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ADD_INFO)) {
		    	    			AdditionalInformation additionalInfo = new AdditionalInformation();

		    	    			if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ADD_INFO_URL)) {
				    	    		String  additionalInfoURL = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
				    	    		additionalInfo.setAdditionalInformationURL(additionalInfoURL);
				    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    			}

			    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_ADD_INFO_URL_DESC)) {
				    	    		String URLDesc = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
				    	    		additionalInfo.setURLDescription(URLDesc);
				    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
			    	    		}
			    	    		fundingOppModSynopsis.setAdditionalInformation(additionalInfo);	
		    	    			updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		    	    		
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AGENCY_CONTACT)) {
			    	    		List <AgencyContact> agencyContactsList = new ArrayList <AgencyContact>(3);
		    	    			do {
		    	    				AgencyContact agencyContact = new AgencyContact();
			        				
			        				if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AGENCY_CONTACT_DESC)) {
				            			String agencyContactDesc = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
				            			agencyContact.setAgencyContactDescription(agencyContactDesc);
				            			updateAdobeOppVN.toElement( VTDNav.PARENT);
			        				}
			            			
			            			if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AGENCY_EMAIL_ADDR)) {
				            			String agencyContactEmailAddr = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
				            			agencyContact.setAgencyEmailAddress( agencyContactEmailAddr);
				            			updateAdobeOppVN.toElement( VTDNav.PARENT);
			            			}
			            			
			            			if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_AGENCY_EMAIL_DESC)) {
				            			String agencyContactEmailAddrDesc =  updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
				            			agencyContact.setAgencyEmailDescription( agencyContactEmailAddrDesc);
				            			updateAdobeOppVN.toElement( VTDNav.PARENT);
			            			}
			    	    			agencyContactsList.add( agencyContact);
		    	    			}while(updateAdobeOppVN.toElement( VTDNav.NEXT_SIBLING, Globals.SYNOPSIS_AGENCY_CONTACT));
		    	    			fundingOppModSynopsis.getAgencyContact().addAll(agencyContactsList);
		    	    			updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}

		        			if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_MOD_SEND_EMAIL)) {
			    	    		String sendNotifEmail = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setSendChangeNotificationEmail( sendNotifEmail);
			    	    		updateAdobeOppVN.toElement( VTDNav.PARENT);
			        		}
		    				
		    	    		if( updateAdobeOppVN.toElement( VTDNav.FIRST_CHILD, Globals.SYNOPSIS_MOD_EMAIL_COMMENTS)) {
			    	    		String modifComments = updateAdobeOppVN.toNormalizedString2( updateAdobeOppVN.getText());
			    	    		fundingOppModSynopsis.setModificationComments( modifComments);
			        			updateAdobeOppVN.toElement( VTDNav.PARENT);
		    	    		}
		        			fundingOppModSynopsisList.add(fundingOppModSynopsis);
	    				} while (updateAdobeOppVN.toElement( VTDNav.NEXT_SIBLING,Globals.SYNOPSIS_MOD_XML));

	    				info.getFundingOppModSynopsis20().addAll(fundingOppModSynopsisList);
		    			retVal.setUpdateAdobeOpportunityInfo(info);

	    			} else {
		    			String s = "Cannot find any FundingOppModSynopsis_2_0 Element in the input XML [" + oppXmlFilePath + "]";
						throw new Exception( s );
		    		}
	    		} else {
					String s = "Cannot find UpdateAdobeOpportunityInfo Root Element in the input XML [" + oppXmlFilePath + "]";
					throw new Exception( s );
	    		}
	    	} catch ( Exception e ) {
	    		log.error( "Client Exception occurred while parsing input xml [" + oppXmlFilePath + "]: " + e.getMessage() );
	    		log.error( "Check your Input file and make sure it is in compliance with the published schema of the current version.");
	    		throw e;
	    	}// try-catch
			return retVal;
		}


}
