package gov.grants.apply.agency.v2;


import gov.grants.apply.services.agencywebservices_v2.AgencyWebServicesPortType;
import gov.grants.apply.services.agencywebservices_v2.UpdateApplicationInfoRequest;
import gov.grants.apply.services.agencywebservices_v2.UpdateApplicationInfoResponse;



import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.ximpleware.VTDNav;

public class UpdateApplicationInfoClient extends BaseAgencyClient {

	private static final String CLASSNAME = UpdateApplicationInfoClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );

		try {
			UpdateApplicationInfoClient client = new UpdateApplicationInfoClient();
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nTEST FAILED - Exception: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	
	public void makeServiceCall() throws Exception {
		try {
			String ggTrackingNum = getArgMap().get( Globals.GG_TRACKING_NUM_CMD_LINE_KEY );
			log.debug( "ggTrackingNum: " + ggTrackingNum );
			if ( StringUtils.isBlank( ggTrackingNum ) ) {
				throw new Exception( "gg_tracking_num is missing from command-line arguments" );
			}// if
			
			String configXml = getArgMap().get( Globals.CONFIG_XML_CMD_LINE_KEY );
			loadConfigXmlDoc( configXml );
			configVN.toElement( VTDNav.FIRST_CHILD, CLASSNAME );// navigate to client element
			
			configVN.toElement( VTDNav.FIRST_CHILD, Globals.AGENCY_TRACKING_NUM_ELEMENT_NAME );
			String agencyTrackingNum = configVN.toNormalizedString2( configVN.getText() );
			log.debug( "agencyTrackingNum: " + agencyTrackingNum );
			
			configVN.toElement( VTDNav.PARENT );
			configVN.toElement( VTDNav.FIRST_CHILD, Globals.AGENCY_NOTES_ELEMENT_NAME );
			String agencyNotes = configVN.toNormalizedString2( configVN.getText() );
			log.debug( "agencyNotes: " + agencyNotes );
			
			if ( StringUtils.isBlank( agencyTrackingNum ) && StringUtils.isBlank( agencyNotes ) ) {
				throw  new Exception( "agency-tracking-num or agency-notes is required in the config XML" );
			}// if
			
			// Create request Objects and populate them with input params.
			UpdateApplicationInfoRequest request = new UpdateApplicationInfoRequest();
			request.setGrantsGovTrackingNumber(ggTrackingNum);
			if ( StringUtils.isNotBlank( agencyTrackingNum ) ) {
				request.setAssignAgencyTrackingNumber( agencyTrackingNum);
			}// if
			
			if ( StringUtils.isNotBlank( agencyNotes ) ) {
				request.setSaveAgencyNotes( agencyNotes );
			}// if
			
			AgencyWebServicesPortType port = getAgencyPort();
			log.debug( "port: " + port );
			log.debug( "Make service call" );
			UpdateApplicationInfoResponse response = port.updateApplicationInfo(request);
			log.debug( "Response Object received: " + response.toString());
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			throw e;
		}// try-catch
		
	}// makeServiceCall
	
}
